package com.dayou.entity;
import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;
import com.dayou.common.BaseEntity;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import lombok.Data;
import lombok.EqualsAndHashCode;
import com.dayou.annotation.ExcelSheet;
import com.dayou.annotation.ExportCell;
import com.dayou.annotation.ImportCell;
/**
 * <p>
 * 资产产品表
 * </p>
 *
 * @author GouGengquan
 * @since 2024-03-28
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ExcelSheet(sheetName = "资产产品表")
public class AssetsProduction extends BaseEntity {

    private static final long serialVersionUID=1L;

    /**
     * 业务id(资产id)
     */
    private Long businessId;

    /**
     * 产品号
     */
    @ImportCell
    @ExportCell(columnName = "产品号")
    private String productionNo;

    /**
     * 产品类型(意见书，报告)
     */
    @ImportCell
    @ExportCell(columnName = "产品类型(意见书，报告)")
    private String productionType;

    /**
     * 签字人（产品类型为报告时，签字人必填）
     */
    @ImportCell
    @ExportCell(columnName = "签字人（产品类型为报告时，签字人必填）")
    private String signatory;

    /**
     * 委托方名称
     */
    @ImportCell
    @ExportCell(columnName = "委托方名称")
    private String clientName;

    /**
     * 委托方电话
     */
    @ImportCell
    @ExportCell(columnName = "委托方电话")
    private String clientTel;

    /**
     * 评估基准日
     */
    @ImportCell
    @ExportCell(columnName = "评估基准日")
    private LocalDate valuationBasisDate;

    /**
     * 评估面积
     */
    @ImportCell
    @ExportCell(columnName = "评估面积")
    private BigDecimal evaluateAcreage;

    /**
     * 评估总价
     */
    @ImportCell
    @ExportCell(columnName = "评估总价")
    private BigDecimal evaluateAmount;

    /**
     * 评估单价
     */
    @ImportCell
    @ExportCell(columnName = "评估单价")
    private BigDecimal evaluatePrice;

    /**
     * 产权人
     */
    @ImportCell
    @ExportCell(columnName = "产权人")
    private String owner;

    /**
     * 产品打印数量
     */
    @ImportCell
    @ExportCell(columnName = "产品打印数量")
    private Integer printCount;

    /**
     * 产品出具日期
     */
    @ImportCell
    @ExportCell(columnName = "产品出具日期")
    private LocalDate createProductionDate;

    /**
     * 特殊情况说明
     */
    @ImportCell
    @ExportCell(columnName = "特殊情况说明")
    private String comment;

    /**
     * 库存状态（0：已入库  1：已出库）
     */
    @ImportCell
    @ExportCell(columnName = "库存状态（0：已入库  1：已出库）")
    private Boolean repertoryState;

    /**
     * 入库时间
     */
    @ImportCell
    @ExportCell(columnName = "入库时间")
    private LocalDateTime repertoryInTime;

    /**
     * 出库时间
     */
    @ImportCell
    @ExportCell(columnName = "出库时间")
    private LocalDateTime repertoryOutTime;

    /**
     * 电子文档路经
     */
    @ImportCell
    @ExportCell(columnName = "资料路由信息(json)")
    private String filePath;

    /**
     * 归档日期
     */
    @ImportCell
    @ExportCell(columnName = "归档日期")
    private Date saveFileDate;

    /**
     * 送达状态
     */
    @ImportCell
    @ExportCell(columnName = "送达状态")
    private Boolean delivery;

    /**
     * 二维码路径
     */
    private String qrCode;

    /**
     * 送达日期
     */
    @ImportCell
    @ExportCell(columnName = "送达日期")
    private LocalDateTime deliveryDate;

    /**
     * 项目名称
     */
    private String assetsName;

    /**
     * 评估对象id集合
     */
    @TableField(exist = false)
    private List<Long> targetIdList;


}
